unit ufrValues;

interface

uses 
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, IniFiles;

type
  TfrValues = class(TFrame)
    cbVal: TComboBox;
    procedure cbValChange(Sender: TObject);
    procedure cbValKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Dclarations prives }
    FLastIndex: Integer;
    FCurrentValue: Single;
    FSectionName: String;
    procedure AddCurrentValue(aValue: Single);
    procedure SetSectionName(aName: String);
  public
    { Dclarations publiques }
    constructor Create(AOwner: TComponent); override;
    procedure LoadIni(aFile: TIniFile);
    procedure SaveIni(aFile: TIniFile);
    property SectionName: String write SetSectionName;
    property CurrentValue: Single read FCurrentValue write AddCurrentValue;
  end;

implementation

{$R *.DFM}

constructor TfrValues.Create(AOwner: TComponent);
begin
  inherited;
  cbVal.ItemIndex:= 0;
  FLastIndex:= cbVal.ItemIndex;
  FCurrentValue:= 0;
  FSectionName:= 'Values';
end;

procedure TfrValues.SetSectionName(aName: String);
begin
  FSectionName:= aName;
end;

procedure TfrValues.AddCurrentValue(aValue: Single);
begin
  cbVal.Text:= FloatToStrF(aValue, ffGeneral, 1, 3);
  cbValChange(Self);
end;

procedure TfrValues.LoadIni(aFile: TIniFile);
var
  aCount, i: Integer;
begin
  aCount:= aFile.ReadInteger(FSectionName, 'Count', 0);
  if aCount > 0 then begin
    cbVal.Clear;
    for i:= 0 to aCount - 1 do
      cbVal.Items.Add(aFile.ReadString(FSectionName, 'Value' + IntToStr(i), '0'));
    cbVal.ItemIndex:= aFile.ReadInteger(FSectionName, 'CurrentIndex', 0);
  end;
  cbValChange(Self);
end;

procedure TfrValues.SaveIni(aFile: TIniFile);
var
  i: Integer;
begin
  aFile.EraseSection(FSectionName);
  aFile.WriteInteger(FSectionName, 'Count', cbVal.Items.Count);
  for i:= 0 to cbVal.Items.Count - 1 do
    aFile.WriteString(FSectionName, 'Value' + IntToStr(i), cbVal.Items[i]);
  i:= cbVal.ItemIndex;
  if i < 0 then i:= 0;
  aFile.WriteInteger(FSectionName, 'CurrentIndex', i);
end;

procedure TfrValues.cbValChange(Sender: TObject);
var
  aVal: Single;
  aCode, aPos: Integer;
  aSt: String;
begin
  aSt:= Trim(cbVal.Text);
  aPos:= cbVal.SelStart;
  if cbVal.ItemIndex >= 0 then begin
    FLastIndex:= cbVal.ItemIndex;
  end;
  Val(aSt, aVal, aCode);
  if aCode = 0 then begin
    cbVal.Font.Color:= clWindowText;
    cbVal.Items[FLastIndex]:= aSt;
    cbVal.ItemIndex:= FLastIndex;
    cbVal.SelStart:= aPos;
    cbVal.SelLength:= 0;
    FCurrentValue:= aVal;
  end else
    cbVal.Font.Color:= clRed;
end;

procedure TfrValues.cbValKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    VK_RETURN: begin
      Inc(FLastIndex);
      cbVal.Items.Insert(FLastIndex, '');
      cbVal.ItemIndex:= FLastIndex;
    end;
    VK_BACK: begin
      if (cbVal.Items.Count > 1) and (cbVal.Text = '') then begin
        cbVal.Font.Color:= clWindowText;
        cbVal.Items.Delete(FLastIndex);
        if FLastIndex > 0 then
          Dec(FLastIndex);
        cbVal.Text:= cbVal.Items[FLastIndex];
      end;
    end;
  end;
end;

end.
