unit uSignalIn;

interface

uses
  Classes, IniFiles, uWaveControl, uWaveIn, uCom, uFFT;
const
  cnPolN = 15;
type
  TComArray = array[0..MaxInt div SizeOf(Com) - 1] of Com;
  pComMask = ^TComArray;
  TPol = array [0..cnPolN] of Extended;
  TMat = array [0..cnPolN, 0..cnPolN] of Extended;
  enSignalMode = (smAdjustment, smCalibration, smAcquisition, smRaw);
  TBoolProc = procedure(aValue: Boolean) of object;
  TRealProc = procedure(aValue: Single) of object;
  TSignalIn = class(TSyncIn)
  private
    FMode: enSignalMode;
    FHarmId, FHarmdWin: Integer;
    FHarmVal: array [0..1] of Com;
    FStrProc: TGetStrProc;
    FOnSaturation: TBoolProc;
    FSetAmp: TRealProc;
    FChanCal: array of Com;
    FFirstCall: Boolean;
    FSat: array [0..1] of Single;
    procedure AllignCalibration;
  protected
    procedure AsyncProc(aData: TWaveData); override;
  public
    Min, Max: Integer;
    constructor Create;
    procedure DoAdjustement(aFreq: Single; aStrProc: TGetStrProc);
    procedure DoCalibration(aMinFreq, aMaxFreq: Single);
    procedure DoAcquisition(aRawMode: Boolean);
    procedure ProcZLCRMeter(aData: TWaveData; aMinFreq, aMaxFreq, aRref,
                            aCin, aCrl, aLin, aRin: Single;
                            aSwap, aCalcLCR, aUI, aParallel: Boolean; aStrProc: TGetStrProc);
    procedure Transfer(aData: TWaveData; aMinFreq, aMaxFreq: Single; aSwap: Boolean);
    property OnSaturation: TBoolProc write FOnSaturation;
    property SetAmp: TRealProc write FSetAmp;
    function LoadIni(aFile: TIniFile): Boolean;
    procedure SaveIni(aFile: TIniFile);
  end;

implementation

uses
  Sysutils, Math;

const
  csSection = 'Calibration';

constructor TSignalIn.Create;
var
  i: Integer;
begin
  inherited;
  for i:= 0 to 1 do
    FHarmVal[i]:= Com1;
  FMode:= smAcquisition;
  FOnSaturation:= nil;
  FStrProc:= nil;
  for i:= 0 to 1 do 
    FSat[i]:= 0.9;
  FChanCal:= nil;
  Min:= 0;
  Max:= 0; 
end;

procedure TSignalIn.AsyncProc(aData: TWaveData);
var
  i, j, iMin, iMax: Integer;
  aNewHarmVal: array [0..1] of Com;
  aVal, aNorm, aFreq: Single;
  aSt: String;
  aRat: Com;
  aMaxAmpl: array [0..1] of Single;
  aSp0, aSp1: pComMask;
  aSaturation: Boolean;
  aLeft: Boolean;
begin
  for i:= 0 to 1 do begin
    aMaxAmpl[i]:= Abs(aData[i,0]);
    for j:= 1 to High(aData[i]) do
      if Abs(aData[i,j]) > aMaxAmpl[i] then
        aMaxAmpl[i]:= Abs(aData[i,j]);
    WaveFFT(@aData[i,0], Length(aData[i]), True);
  end;
  aSaturation:= (aMaxAmpl[0] >= FSat[0]) or
                (aMaxAmpl[1] >= FSat[1]);
  aSp0:= @aData[0,0];
  aSp1:= @aData[1,0];

  case FMode of

    smAdjustment:
      begin
        aSt:= 'Quality:';
        for i:= 0 to 1 do begin
          aNewHarmVal[i]:= cCom(aData[i, 2*FHarmId]);
          aVal:= 0;
          j:= FHarmId*2;
          while j < Length(aData[i]) div 2 - 1 do begin
            aVal:= aVal + cMod2(cCom(aData[i, 2*j]));
            j:= j + FHarmId;
          end;
          if aVal <> 0 then
            aSt:= aSt + ' ' + FloatToStrF(
              10 * Log10(cMod2(aNewHarmVal[i])/aVal), ffFixed, 0, 0)
          else
            aSt:= aSt + '??';
        end;
        if not FFirstCall then begin
          aVal:= 0;
          for i:= 0 to 1 do begin
            if FSat[i] < aMaxAmpl[i] then
              FSat[i]:= aMaxAmpl[i];
            aRat:= cDiv(aNewHarmVal[i], FHarmVal[i]);
            if aRat.x <> 0 then
              aVal:= aVal + aRat.y / aRat.x
            else
              aVal:= aVal - 100;
          end;
           aSt:= aSt +  Chr(13) + Chr(10) + 'Async: ' +
             FloatToStrF(aVal, ffFixed, 0, 3);
        end;
        for i:= 0 to 1 do
          FHarmVal[i]:= aNewHarmVal[i];
        if Assigned(FStrProc) then
          FStrProc(aSt);
      end;

    smCalibration:
      //if not FFirstCall then
      begin
        aNorm:= 0;
        for i:= 1 to Length(aData[0]) div 2 - 1 do begin
          aVal:= Sqrt(cMod2(aSp0[i])) + Sqrt(cMod2(aSp1[i]));
          if aVal > aNorm then aNorm:= aVal;
        end;
        aLeft:= True;
        iMin:= 1;
        iMax:= Length(aData[0]) div 2 - 1;
        for i:= 1 to Length(aData[0]) div 2 - 1 do begin
          aVal:= 4*(Sqrt(cMod2(aSp0[i])) + Sqrt(cMod2(aSp1[i])));
          if aLeft then begin
            iMin:= i;
            if aVal > aNorm then
              aLeft:= False;
          end else 
            if aVal > aNorm then
              iMax:= i
            else
              Break;
        end;
        aFreq:= Sampling / DataCount;
        Min:= Round(iMin*aFreq);
        Max:= Round(iMax*aFreq);
        aRat:= Com0;
        for i:= 0 to iMin - 1 do begin
          aSp0[i]:= aRat;
          FChanCal[i]:= aRat;
        end;
        for i:= iMax + 1 to Length(aData[0]) div 2 - 1 do begin
          aSp0[i]:= aRat;
          FChanCal[i]:= aRat;
        end;

//        for i:= 1 to Length(aData[0]) div 2 - 1 do begin
        for i:= iMin to iMax do begin
          aRat:= cDiv(aSp0[i], aSp1[i]);
          //aRat:= cAmPh(aRat);
          aSp0[i]:= aRat;
          FChanCal[i]:= aRat;
        end;

        for i:= 0 to Length(aData[0]) div 2 - 1 do
          aSp1[i]:= FChanCal[i];
        {
        for i:= 1 to Length(aData[0]) div 2 - 1 do begin
          aSp1[i]:= cSub(FChanCal[i], aSp0[i]);
          aSp0[i]:= aSp1[i];
        end;
            }
        if aSaturation then begin
          if Assigned(FSetAmp) and aSaturation then
            FSetAmp(0.90);
          aSt:= 'Wait...'
        end else
          aSt:= 'Press Stop'+  Chr(13) + Chr(10)
              + 'to finish.';
        if Assigned(FStrProc) then
          FStrProc(aSt);
     end;

    smAcquisition:
      begin
        {
        aRat:= Com0;
        for i:= 0 to 1 do if FSat[i] > 0 then
          aRat.u[i]:= aMaxAmpl[i] / FSat[i];
        if aRat.x < aRat.y then
          aRat.x:= aRat.y;
        if (aRat.x > 0) and (aRat.x < 0.7) and Assigned(FSetAmp) then
             FSetAmp(0.7/aRat.x);
        }
        for i:= 1 to Length(aData[0]) div 2 - 1 do begin
          aSp1[i]:= cMul(aSp1[i], FChanCal[i]);
        end;
      end;

    smRaw:
      begin
        aRat:= Com0;
        for i:= 0 to 1 do if FSat[i] > 0 then
          aRat.u[i]:= aMaxAmpl[i] / FSat[i];
        if aRat.x < aRat.y then
          aRat.x:= aRat.y;
        if (aRat.x > 0) and (aRat.x < 0.7) and Assigned(FSetAmp) then
             FSetAmp(0.7/aRat.x);
        aNorm:= 100/(Length(aData[0]) div 2);
        for i:= 1 to Length(aData[0]) div 2 - 1 do begin
          aSp0[i]:= cMuR(aSp0[i], aNorm);
          aSp1[i]:= cMuR(aSp1[i], aNorm);
        end;
      end;
  end;

{ if (FMode <> smAdjustment) then begin
    if Assigned(FOnSaturation) then
      FOnSaturation(aSaturation);
    if Assigned(FSetAmp) and aSaturation then
      FSetAmp(0.9);
  end; }
  FFirstCall:= False;
end;

procedure TSignalIn.DoAdjustement(aFreq: Single; aStrProc: TGetStrProc);
begin
  FHarmId:= FWaveIn.GetHarmonic(aFreq, DataCount);
  FStrProc:= aStrProc;
  FMode:= smAdjustment;
  FFirstCall:= True;
  FillChar(FSat, SizeOf(FSat), 0);
end;

procedure TSignalIn.AllignCalibration;
var
  i: Integer;
begin
  i:= DataCount div 2;
  if Length(FChanCal) <> i then begin
    SetLength(FChanCal, i);
    for i:= 0 to High(FChanCal) do
      FChanCal[i]:= Com1;
  end;
end;

procedure TSignalIn.DoCalibration(aMinFreq, aMaxFreq: Single);
var
  aMin, aMax: Integer;
begin
  FMode:= smCalibration;
  aMin:= FWaveIn.GetHarmonic(aMinFreq, DataCount);
  aMax:= FWaveIn.GetHarmonic(aMaxFreq, DataCount);
  FHarmId:= (aMin + aMax) div 2;
  FHarmdWin:= FHarmId - aMin;
  AllignCalibration;
end;

procedure TSignalIn.DoAcquisition(aRawMode: Boolean);
begin
  if aRawMode then
    FMode:= smRaw
  else begin
    FMode:= smAcquisition;
    AllignCalibration;
  end;
end;

function ValToStr(aValue: Extended): String;
const cPrecision = 4;
      cDigits = 0;
begin
  if aValue = 0 then begin
    Result:= '0 ';
    Exit;
  end;  
  if Abs(aValue) > 1E3 then
  if Abs(aValue) > 1E6 then
    Result:= FloatToStrF(aValue * 1E-6, ffGeneral, cPrecision, cDigits) + ' M'
  else
    Result:= FloatToStrF(aValue * 1E-3, ffGeneral, cPrecision, cDigits) + ' k'
  else
  if Abs(aValue) < 1.0 then
  if Abs(aValue) < 1E-3 then
  if Abs(aValue) < 1E-6 then
  if Abs(aValue) < 1E-9 then
    Result:= FloatToStrF(aValue * 1E12, ffGeneral, cPrecision, cDigits) + ' p'
  else
    Result:= FloatToStrF(aValue * 1E9,  ffGeneral, cPrecision, cDigits) + ' n'
  else
    Result:= FloatToStrF(aValue * 1E6,  ffGeneral, cPrecision, cDigits) + ' '
  else
    Result:= FloatToStrF(aValue * 1E3,  ffGeneral, cPrecision, cDigits) + ' m'
  else
    Result:= FloatToStrF(aValue,        ffGeneral, cPrecision, cDigits) + ' ';
end;

procedure TSignalIn.ProcZLCRMeter(aData: TWaveData;
  aMinFreq, aMaxFreq, aRref, aCin, aCrl, aLin, aRin: Single;
  aSwap, aCalcLCR, aUI, aParallel: Boolean; aStrProc: TGetStrProc);
var
  i, aMin, aMax, aNL, aNC, aNTg, aNR: Integer;
  aLx2, aLxy, aL, aCx2, aCxy, aC,
  aR, aDen, aBaseFreq, aOmega, aTg: Single;
  aIn0, aIn1, aOut0, aOut1: pComMask;
  aSt: String;
  aTmp, aZRef: Com;
begin
  if aSwap then begin
    aIn1:= @aData[0,0];
    aIn0:= @aData[1,0];
  end else begin
    aIn0:= @aData[0,0];
    aIn1:= @aData[1,0];
  end;
  aOut0:= @aData[0,0];
  aOut1:= @aData[1,0];
  aMin:= FWaveIn.GetHarmonic(aMinFreq, DataCount);
  aMax:= FWaveIn.GetHarmonic(aMaxFreq, DataCount);
  aBaseFreq:= 2*Pi*Sampling / DataCount;
  aL:= 0;
  aC:= 0;
  aR:= 0;
  aTg:= 0;
  aNL:= 0;
  aNC:= 0;
  aLx2:= 0;
  aLxy:= 0;
  aCx2:= 0;
  aCxy:= 0;
  aNTg:= 0;
  aNR:= 0;
  for i:= aMin to aMax do begin
    aOmega:= aBaseFreq * i;
    aDen:= aCin*aRref*aOmega;
    aDen:= aDen*aDen + 1;
    aZRef.x:= aRref/aDen;
    aZRef.y:= -aRref*aRref*aCin*aOmega/aDen;
    aTmp:= cDiv(aIn0[i], aIn1[i]);
    if not aUI then
      aTmp.x:= aTmp.x - 1;
    aTmp.y:= -aTmp.y;
    aTmp:= cMul(aZRef, aTmp);
    aZRef.x:= 1 + aTmp.y*aOmega*aCrl;
    aZRef.y:= -aTmp.x*aOmega*aCrl;
    aTmp:= cDiv(aTmp, aZRef);
    aTmp.x:= aTmp.x - aRin;
    aTmp.y:= aTmp.y - aOmega*aLin;
    if aParallel then begin
      aTmp:= cInv(aTmp);
      if aTmp.x <> 0 then
        aTmp.x:= 1/aTmp.x;
      if aTmp.y <> 0 then
        aTmp.y:= -1/aTmp.y;
    end;
    aOut0[i]:= aTmp;
    if aCalcLCR then begin
      //if aOut0[i].x > 0 then begin
        aR:= aR + aOut0[i].x;
        aNR:= aNR + 1;
      //end;
      if aOut0[i].y <> 0 then begin
        aTg:= aTg + aOut0[i].x / aOut0[i].y;
        aNTg:= aNTg + 1;
      end;
      aLx2:= aLx2 + aOmega * aOmega;
      aLxy:= aLxy + aOut0[i].y * aOmega;
      aCx2:= aCx2 + 1 / (aOmega*aOmega);
      aCxy:= aCxy - aOut0[i].y / aOmega;
      if aOut0[i].y > 0 then begin
        aNL:= aNL + 1;
      end else begin
        aNC:= aNC + 1;
      end;
    end;
  end;
  if not aCalcLCR then
    Exit;
  if aNL > aNC then begin
    aL:= aLxy / aLx2;
    aSt:= 'Lx= ' + ValToStr(aL) + 'H';
  end else
    if aCxy <> 0 then begin
      aC:= aCx2 / aCxy;
      aSt:= 'Cx= ' + ValToStr(aC) + 'F';
    end else begin
      aC:= 1E6;
      aSt:= 'Cx= ??';
    end;
  //if aR < 0 then
  //  aR:= 0;  
  if aNR > 0 then
    aR:= aR / aNR
  else
    aR:= 0;  
  aSt:= aSt + Chr(13) + Chr(10) +
    'R= ' + ValToStr(aR) + 'Ohm';
  if aNTg > 0 then begin
    aTg:= Abs(aTg / aNTg);
    aSt:= aSt + Chr(13) + Chr(10) +
      'tg= ' + FloatToStrF(aTg, ffExponent, 2, 2);
  end else
    aSt:= aSt + Chr(13) + Chr(10) +
      'tg= ??';
  for i:= aMin to aMax do begin
    aOmega:= aBaseFreq * i;
    aOut1[i].x:= aR;
    if aNL > aNC then
      aOut1[i].y:= aOmega * aL
    else
      aOut1[i].y:= -1 / (aOmega * aC);
  end;
  if Assigned(aStrProc) then
    aStrProc(aSt);
end;

procedure TSignalIn.Transfer(aData: TWaveData; aMinFreq, aMaxFreq: Single;
                             aSwap: Boolean);
var
  i, aMin, aMax: Integer;
  aIn0, aIn1, aOut0: pComMask;
begin
  if aSwap then begin
    aIn1:= @aData[0,0];
    aIn0:= @aData[1,0];
  end else begin
    aIn0:= @aData[0,0];
    aIn1:= @aData[1,0];
  end;
  aOut0:= @aData[0,0];
  aMin:= FWaveIn.GetHarmonic(aMinFreq, DataCount);
  aMax:= FWaveIn.GetHarmonic(aMaxFreq, DataCount);
  for i:= aMin to aMax do begin
    // aOut0[i].x:= Sqrt(cMod2(aIn0[i]));
    // aOut0[i].y:= Sqrt(cMod2(aIn1[i]));
    aOut0[i]:= cDiv(aIn1[i], aIn0[i]);
  end;
end;

function TSignalIn.LoadIni(aFile: TIniFile): Boolean;
var
  i: Integer;
  aCalFile: TFileStream;
begin
  Result:= False;
  for i:= 0 to 1 do
    FSat[i]:= aFile.ReadFloat(csSection, 'Saturation' + IntToStr(i), FSat[i]);
  Min:= aFile.ReadInteger(csSection, 'fMin', Min);
  Max:= aFile.ReadInteger(csSection, 'fMax', Max);
  try
    aCalFile:= TFileStream.Create(ChangeFileExt(Paramstr(0),'.cal'), fmOpenRead);
  except
    Exit;
  end;
  aCalFile.Read(i, SizeOf(i));
  SetLength(FChanCal, i);
  aCalFile.Read(FChanCal[0], SizeOf(FChanCal[0])*i);
  aCalFile.Free;
  Result:= i > 0;
end;

procedure TSignalIn.SaveIni(aFile: TIniFile);
var
  i: Integer;
  aCalFile: TFileStream;
begin
  for i:= 0 to 1 do
    aFile.WriteFloat(csSection, 'Saturation' + IntToStr(i), FSat[i]);
  aFile.WriteInteger(csSection, 'fMin', Min);
  aFile.WriteInteger(csSection, 'fMax', Max);
  aCalFile:= TFileStream.Create(ChangeFileExt(Paramstr(0),'.cal'), fmCreate);
  i:= Length(FChanCal);
  aCalFile.Write(i, SizeOf(i));
  aCalFile.Write(FChanCal[0], SizeOf(FChanCal[0])*i);
  aCalFile.Free;
end;

end.
